#include <stdint.h>
#include <stdbool.h>

// C_Meter Click
sbit C_METER_RST 	at PORTA6_bit;
sbit C_METER_RST_DIR 	at DDA6_bit;
sbit C_METER_INT 	at PIND2_bit;
sbit C_METER_INT_DIR 	at DDD2_bit;

void system_setup( void );
void setup_interrupt( void );
void InitTimer1( void );

volatile bool flag;
volatile uint32_t start_val;
volatile uint32_t end_val;
char uart_text[ 40 ];
char final_text[ 40 ];
uint32_t int_count = 0;
uint32_t timer_count_end = 0;
bool sec_flag = false;
volatile float c_cal = 0.0;
volatile float t_meas = 0.0;
volatile float c_meas = 0.0;
volatile float final = 0.0;
volatile float t_cal = 0.0;


void main()
{
    //Local Declarations
    bool calibration_flag = true;


    system_setup();
    setup_interrupt();
    InitTimer1();

    t_cal = 1.0 / 22587.0;
    c_cal = t_cal / 77616.0;

    int_count = 0;

    while(1)
    {

        if( sec_flag )
        {
            //Debugging
            sprintf( uart_text, "Interrupts : %u\r\n", timer_count_end );
            UART1_Write_Text( uart_text );
            //Measurment
            t_meas = 1.0 / timer_count_end;
            c_meas = t_meas / 77616.0;
            final = fabs( c_cal - c_meas );
            //Print out
            UART1_Write_Text( "Calibrated: " );
            FloatToStr( c_cal, uart_text );
            UART1_Write_Text( uart_text );
            UART1_Write_Text( "\r\n" );
            UART1_Write_Text( "Final: " );
            FloatToStr( final, final_text );
            UART1_Write_Text( final_text );
            UART1_Write_Text( "\r\n" );
            //Reset flags
            sec_flag = false;
            int_count = 0;
            timer_count_end = 0;
        }
    }
}

void system_setup( void )
{
    //GPIO
    C_METER_INT_DIR = 0;
    C_METER_RST_DIR = 1;

    //RST Toggle
    C_METER_RST = 0;
    Delay_ms(50);
    C_METER_RST = 1;

    //UART
    UART1_Init( 9600 );
    UART1_Write_Text( "UART Initialized\r\n" );

}

void setup_interrupt( void )
{
    GICR |= ( 1 << 6 );
    MCUCR |= 3;
    SREG |= ( 1 << 7 );
}

void interrupt() iv IVT_ADDR_INT0 ics ICS_AUTO 
{
    int_count++;
}

void InitTimer1(){
  SREG_I_bit = 1;
  TCCR1A = 0x80;
  TCCR1B = 0x0C;
  OCR1AH = 0x7A;
  OCR1AL = 0x11;
  OCIE1A_bit = 1;
}

void Timer1Overflow_ISR() org IVT_ADDR_TIMER1_COMPA {
  //Enter your code here
  timer_count_end = int_count;
  int_count = 0;
  sec_flag = true;
}
